#!/bin/bash
APPNAME=deCONZ

# prevent SIGILL signal in openssl library
EXTRA_OPTIONS=${DECONZ_OPTS:-}

UPDATE_FW_SCRIPT="/var/tmp/deCONZ-update-firmware.sh"

#BASEURL="http://www.dresden-elektronik.de/rpi"

FWDIROLD="/home/${USER}/raspbee_firmware"
OTAUDIR="/home/${USER}/otau"

# set date behind a proxy (no NTP/UDP)
# sudo date -s "$(curl --head -i - duckduckgo.com | grep '^Date:' | cut -d' ' -f3-6)Z"

#check if deCONZ already running
pidof deCONZ
if [ $? -eq "0" ]; then
	echo "deCONZ already running"
	exit 1
fi

# auto configure system
OS_ID=$(cat /etc/os-release | sed -n 's/^ID=\(.*\)/\1/p')
OS_VERSION=$(cat /etc/os-release | sed -n 's/^VERSION="[0-9]\s(\(.*\))"/\1/p')
NEED_REBOOT=0

if [ "$OS_ID" == "raspbian" ] && [ "$OS_VERSION" == "wheezy" ]; then
	# TODO
	echo "configure wheezy"
fi

if [ "$OS_ID" == "raspbian" ] && [ "$OS_VERSION" == "jessie" ]; then
	# check dependencies
	deps=("libqt5core5a" "libqt5network5" "libqt5widgets5" "libqt5gui5" "libqt5serialport5" "libqt5websockets5" "libqt5sql5" "wiringpi")
	for i in "${deps[@]}"; do
		dpkg -l $i 2> /dev/null | grep '^ii' > /dev/null

		if [ $? -ne 0 ]; then			
			echo "missing dependency $i"
			exit 3
		fi
	done

    # on Raspbian Jessie RaspBee can co exist with bluetooth
    egrep -q -e "^enable_uart=1" /boot/config.txt
    if [ $? -eq 1 ]; then
        sudo sed -i s/^enable_uart=.*//g /boot/config.txt
        sudo sh -c 'echo "enable_uart=1" >> /boot/config.txt'
        NEED_REBOOT=1
    fi

    egrep -q -e "console=serial0" /boot/cmdline.txt
    if [ $? -eq 0 ]; then
        echo "change /boot/cmdline.txt"
        sudo sed -i s/console=serial0,115200//g /boot/cmdline.txt
        NEED_REBOOT=1
    fi

    egrep -q -e "^dtoverlay=pi3-miniuart-bt-overlay" /boot/config.txt
    if [ $? -eq 1 ]; then
        sudo sed -i s/^dtoverlay=pi3-disable-bt.*//g /boot/config.txt
        sudo sh -c 'echo "dtoverlay=pi3-miniuart-bt-overlay" >> /boot/config.txt'
        NEED_REBOOT=1
    fi
fi

if [ "$OS_ID" == "ubuntu" ]; then
	# check dependencies
	DOSETUP=0
	NOGROUP=0

	deps=("libqt5core5a" "libqt5network5" "libqt5widgets5" "libqt5gui5" "libqt5serialport5")
	for i in "${deps[@]}"; do
		dpkg -l $i 2> /dev/null | grep '^ii' > /dev/null

		if [ $? -ne 0 ]; then
			DOSETUP=1
		fi
	done

	groups | grep &>/dev/null 'dialout'
	if [ $? -ne 0 ]; then
		NOGROUP=1
	fi

	if [ $DOSETUP -eq 1 ] || [ $NOGROUP -eq 1 ]; then

		zenity --title="deCONZ Setup" --question --text="Some missing dependencies must be install as super user.\nProceed?"

		if [ $? -eq 0 ]; then

			pkexec /usr/bin/deCONZ-setup.sh $USER
	
			case $? in
			0)
				# OK
				;;
			3)
				zenity --title="deCONZ Setup" --error --text="Failed to install dependencies\n ${deps[@]}"
				exit 1
				;;
			*)
				zenity --title="deCONZ Setup" --error --text="An error occured during setup $?"
				exit 1
				;;
			esac

			if [ $NOGROUP -eq 1 ]; then
				# need relogin
#				zenity --title="deCONZ Setup" --info --text="Modified system configuration to provide to access the USB dongle.\nPlease logout and login again to activate the changes."
				# replace process with new one which is part of the group (whole script will be executed again)
				exec sg dialout deCONZ-autostart.sh
				#exit 1
			fi
		else
			exit 1
		fi
	fi
fi

if [ $NEED_REBOOT -eq 1 ]; then
    # interactive shell?
    if [ -z "$PS1" ]; then
        sudo reboot
    else
        echo "please reboot the system with $ sudo reboot"
    fi

    exit 1
fi

if [ -d "$FWDIROLD" ]; then
	echo "remove old firmware directory"
	rm -rf "$FWDIROLD"
fi

# sleep for some seconds to give the system the chance to connect to the internet 
# sleep 10

# create otau directory if not exist
if [ ! -d "$OTAUDIR" ]; then
    mkdir -p ${OTAUDIR}
fi

#check if deCONZ OTAU download script already running
DLPID=`pidof deCONZ-otau-dl.sh`
if [ $? -eq "0" ]; then
	echo "deCONZ-otau-dl.sh already running $DLPID"
else
	deCONZ-otau-dl.sh &
	DLPID=$!
fi

function finish {
	echo "shutdown deCONZ"
	
	kill -9 $DLPID
#	if [ "$OS_ID" == "raspbian" ]; then
		#echo "resetting device"
		#sudo GCFFlasher -r
#	fi
}

trap finish EXIT

# check for setcap / getcap
if [ "$OS_ID" == "raspbian" ]; then
	dpkg -l libcap2-bin > /dev/null 2>&1

	if [ $? -ne 0 ]; then
	    sudo apt-get install --yes libcap2-bin
	fi
fi

while [ 1 ]
do
	# provide service on port 80
	which setcap
	OPTIONS="--auto-connect=1 --dbg-error=1"

	if [ $? -eq "0" ] && [ "$OS_ID" == "raspbian" ]; then
		# assume getcap also exists
		caps=`getcap /usr/bin/deCONZ`

		if [[ "$caps" == *cap_net_bind_service+ep* ]]; then
			echo capabilities ok
		else
			sudo setcap cap_net_bind_service=+ep  /usr/bin/deCONZ
			echo set capability to open ports lower 1024
		fi

		OPTIONS+=" --http-port=80"
	else
		OPTIONS+=" --http-port=8080"
	fi

	# print backtrace on SEGV
	#export LD_PRELOAD=/usr/lib/debug/lib/arm-linux-gnueabihf/libSegFault.so

	$APPNAME $OPTIONS $EXTRA_OPTIONS
	RC=$?
	echo "deCONZ - exit code ${RC}"

	case "$RC" in
	0) # Normal exit
		exit 0
		;;
	40) # Download and install updates
		deCONZ-update.sh
		;;
	41) # Restart this script in a new process
		exec $@
		;;
	42) # Download and install updates (beta channel)
		deCONZ-update.sh -c beta
		;;
	43) # Restart system
		sudo reboot
		exit 0
		;;
	44) # Shutdown system
		sudo halt
		exit 0
		;;
	45) # Download and install updates (alpha channel)
		# not yet supported just do a beta channel update
		# deCONZ-update.sh -c alpha
		deCONZ-update.sh -c beta
		;;
	46) # Update firmware only
		if [ -e "$UPDATE_FW_SCRIPT" ]; then
			sh $UPDATE_FW_SCRIPT
			# cleanup
			echo "delete $UPDATE_FW_SCRIPT"
			rm $UPDATE_FW_SCRIPT
		else
			echo "$UPDATE_FW_SCRIPT not found"
		fi
		;;
	126) # Permission problem or command is not executable
		finish
		exit 1
		;;
	127) # Command not found (try to install)
		deCONZ-update.sh
		if [ $? -ne 0 ]; then
			finish
			exit 1
		fi
		;;
	130) # CTRL-C, SIGINT (128 + 2)
		finish
		exit 0
		;;
	137) # SIGKILL (128 + 9)
		finish
		exit 0
		;;
	139) # SEGV (128 + 11)
		echo "SEGV received, wait 10s and restart deCONZ"
		sleep 10
		;;
	143) # SIGTERM (128 + 15)
		finish
		exit 0
		;;
	*) echo "exit code $RC not handled"
		;;
	esac
done
